/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/



/** \file Config_clang.h */

#pragma once

#define CLANG_VERSION (__clang_major__ * 10000 + __clang_minor__ * 100 + __clang_patchlevel__)

#define COMPILER_NAME "clang"
#define COMPILER_VERSION CLANG_VERSION

// alignement specification
// ------------------------
#define ALIGNED(type,name,N)  type name[N] __attribute__ ((aligned(CACHE_LINE)))
#define ASSUME_ALIGNED(var)   __builtin_assume_aligned(var, CACHE_LINE)

typedef double aligned_double   __attribute__ ((aligned(CACHE_LINE)));
typedef float  aligned_float    __attribute__ ((aligned(CACHE_LINE)));
typedef int    aligned_int      __attribute__ ((aligned(CACHE_LINE)));
typedef size_t aligned_size_t   __attribute__ ((aligned(CACHE_LINE)));

// some compiler attributes
// ------------------------
#define NOINLINE                __attribute__ ((noinline))
#define ALWAYS_INLINE           __attribute__ ((always_inline))
#define OPENMODE                std::ios::openmode

// C++11 features
// --------------
#if __cplusplus > 199711L
#define HAS_CXX11 1

// __has_feature(cxx_rvalue_references)
#if CLANG_VERSION >= 20900 && !defined(HAS_VARIADIC_TEMPLATES)
  #define HAS_VARIADIC_TEMPLATES 1
#endif

#if CLANG_VERSION >= 30000 && !defined(HAS_ALIAS_TEMPLATES)
  #define HAS_ALIAS_TEMPLATES 1
#endif

#if CLANG_VERSION >= 20900 && !defined(HAS_DECLTYPE)
  #define HAS_DECLTYPE 1
#endif

#if CLANG_VERSION >= 30100 && !defined(HAS_CONSTEXPR)
  #define HAS_CONSTEXPR 1
#endif

#if CLANG_VERSION >= 30000 && !defined(HAS_DELEGATING_CONSTRUCTORS)
  #define HAS_DELEGATING_CONSTRUCTORS 1
#endif

#if CLANG_VERSION >= 30000 && !defined(HAS_RANGE_BASED_FOR)
  #define HAS_RANGE_BASED_FOR 1
#endif

#if CLANG_VERSION >= 30100 && !defined(HAS_INITIALIZER_LISTS)
  #define HAS_INITIALIZER_LISTS 1
#endif

#if CLANG_VERSION >= 30000 && !defined(HAS_OVERRIDE)
  #define HAS_OVERRIDE 1
#endif

#if CLANG_VERSION >= 20900 && !defined(HAS_TYPED_ENUMS)
  #define HAS_TYPED_ENUMS 1
#endif

#if CLANG_VERSION >= 20900 && !defined(HAS_RVALUE_REFERENCES)
  #define HAS_RVALUE_REFERENCES 1
#endif

#endif
