/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


#ifndef EXTENSIONS_REFINE_FUNCTIONS_H
#define EXTENSIONS_REFINE_FUNCTIONS_H

#include <vector>
#include <string>

#include "Global.h"
#include "Mesh.h"
#include "Initfile.h"
#include "operations/functors.hpp"

namespace AMDiS { namespace extensions {
  
  struct PhaseFieldRefineFct : public FunctorBase
  {
    typedef int result_type;
    
    PhaseFieldRefineFct(std::string name, Mesh* mesh)
      : lInner(10),
	lOuter(10),
	lInterface(14),
	minPhase(0.05),
	maxPhase(0.95),
	minOuterPhase(0.001),
	maxOuterPhase(0.999)
    {
      Parameters::get(name + "->level in inner domain", lInner);
      Parameters::get(name + "->level in outer domain", lOuter);
      Parameters::get(name + "->level on interface", lInterface);

      Parameters::get(name + "->min interface value", minPhase);
      Parameters::get(name + "->max interface value", maxPhase);
      Parameters::get(name + "->min outer interface value", minOuterPhase);
      Parameters::get(name + "->max outer interface value", maxOuterPhase);
	
      lInterface-= mesh->getMacroElementLevel();
      lInner-= mesh->getMacroElementLevel();
      lOuter-= mesh->getMacroElementLevel();
    }
    
    inline int operator()(double phase) const
    {
      int result = lOuter;
      if (minPhase < phase && phase < maxPhase)
        result = lInterface;         // auf dem Interface
      else if (phase > minOuterPhase && phase <= minPhase)
        result = std::max(lOuter, static_cast<int>(std::floor((lOuter+lInterface)/2.0)));
      else if (phase < maxOuterPhase && phase >= maxPhase)
        result = std::max(lInner, static_cast<int>(std::floor((lInner+lInterface)/2.0)));
      else if (phase > (minPhase+maxPhase)/2.0)
        result = lInner;
      return result;
    }
    
  protected:
    int lInner;
    int lOuter;
    int lInterface;
    
    double minPhase;
    double maxPhase;
    double minOuterPhase;
    double maxOuterPhase;
  };
    
} }

#endif