/** \file BaseProblem_RB0.h */

#ifndef BASE_PROBLEM_RB0_H
#define BASE_PROBLEM_RB0_H

#include "AMDiS.h"
#include "BaseProblem.h"
// #include "time/ExtendedRosenbrockStationary.h"

namespace AMDiS { namespace base_problems {

///BaseProblems for Rosenbrock time-discretization
class BaseProblem_RB0 : public BaseProblem<RosenbrockStationary>
{
public: // typedefs

  typedef BaseProblem<RosenbrockStationary> super;

// Rosenbrock methods
//_________________________________________________________________________

public:
  
  BaseProblem_RB0(std::string name_, bool createProblem = true) : super(name_, createProblem), minus1(-1.0) {}
  
  Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo) 
  { 
    prob->oneIteration(adaptInfo, toDo);
    
    oldTimestep = adaptInfo->getTimestep();
    oldMeshChangeIdx = getMesh()->getChangeIndex();
  }
  
  void setRosenbrockMethod(RosenbrockMethod *method) { prob->setRosenbrockMethod(method); }
  void setTau(double *ptr) { prob->setTau(ptr); }
  void setTauGamma(double *ptr0, double *ptr1, double *ptr2, double *ptr3)
  {
    prob->setTauGamma(ptr0, ptr1, ptr2, ptr3);
  }  
    
  /// get the i-th stage-solution-vector
  DOFVector<double> *getStageSolution(int i)
  { FUNCNAME("CoupledBaseProblem::getStageSolution(i,j)");
  
    TEST_EXIT(0<=i && i<getNumComponents())("Indices out of range!\n");
    return prob->getStageSolution(i);
  }  
  
  /// get the i-th unVec-vector
  DOFVector<double> *getUnVec(int i) 
  { FUNCNAME("CoupledBaseProblem::getUnVec(i)");
  
    TEST_EXIT(0<=i && i<getNumComponents())("Indices out of range!\n");
    return prob->getUnVec(i);
  }
  
protected:
  double minus1;
};

} }

#endif // BASE_PROBLEM_RB_H