/** \file VacancyPhaseFieldCrystal.h */

#ifndef VACANCY_PHASE_FIELD_CRYSTAL_H
#define VACANCY_PHASE_FIELD_CRYSTAL_H

#include "AMDiS.h"
#include "BaseProblem.h"

namespace AMDiS { namespace base_problems {

/** Phase-field Crystal problem
 */
class VacancyPhaseFieldCrystal : public StandardBaseProblem
{
public: // typedefs

  typedef StandardBaseProblem super;

public:

  VacancyPhaseFieldCrystal(const std::string &name_, bool createProblem = true);
  ~VacancyPhaseFieldCrystal() {};

  virtual void fillOperators();

protected:

  double r;
  double rho;
  
  double two;
  double minus2;
  double three;
  double minus3;
  double four;
  double minus4;
  double eight;
  double minus8;
};


/** \ingroup MainInstat
 * \brief
 * Abstract function to calculate the pure PFC-Energy
 */
class Energy : public BinaryAbstractFunction<double,double,double>
{
  public:
    Energy() : BinaryAbstractFunction<double,double,double>(4) { }

    double operator()(const double &rho, const double &mu) const {
      return -0.25*sqr(sqr(rho)) + 0.5*rho*mu; }
};


class MobilityPfc : public AbstractFunction<double,double>
{
  public:
    MobilityPfc(double density_ = -0.3, double factor_ = 1.0) : 
      AbstractFunction<double,double>(1),
      density(density_),
      factor(factor_),
      delta(1.e-6) { }

    double operator()(const double &rho) const 
    {
      double mobility= abs(rho - 3.0*density)*factor;
      return std::max(mobility, delta);
    }
  
  protected:
    double density;
    double factor;
    double delta;
};

} }

#endif // PHASE_FIELD_CRYSTAL_H
