/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_CH_H
#define AMDIS_PETSC_PRECON_CH_H

#include "solver/PetscSolver.h"

namespace AMDiS {

  using namespace std;

  struct ChData {
    ChData() : matM(PETSC_NULL), matMinusDeltaK(PETSC_NULL) {}
    KSP kspM, kspMpK;
    Mat matM, matMinusDeltaK;
    double delta, eps;
  };

  class PetscPreconCahnHilliard : public PetscPreconditionerNested
  {
  public:
    typedef PetscMatrixNested    MatrixType;
    typedef PetscVectorNested    VectorType;
    
    typedef PetscPreconditionerNested            super;
    typedef PetscPreconCahnHilliard              self;
    typedef PetscRunner<MatrixType, VectorType>  Runner;
    
    struct Creator : CreatorInterfaceName<super>
    {
      virtual ~Creator() {}
      super* create() { return new self(); }
    };
    
    PetscPreconCahnHilliard()
      : super("ch_", "ch"),
	MpK(PETSC_NULL),
	eps(NULL),
	delta(NULL) { }

    void setData(double *epsPtr, double *deltaPtr)
    {
      eps = epsPtr;
      delta = deltaPtr;
    }

    void init(PC pc, const SolverMatrix<Matrix<DOFMatrix*> >& A, const MatrixType& fullMatrix);
    void exit();

  private:
    Mat MpK;
    ChData data;

    double *eps;
    double *delta;
  };

}

#endif // AMDIS_PETSC_PRECON_PFC_H
