/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_PFC_DIAG_H
#define AMDIS_PETSC_PRECON_PFC_DIAG_H

#include "solver/PetscSolver.h"

namespace AMDiS {

  using namespace std;

  struct PfcDiagData {
    KSP kspM, kspMpK, kspS;
    Mat matM, matK;
    double delta, tau;
  };

  class PetscPreconPfcDiag : public PetscPreconditionerNested
  {
  public:
    typedef PetscMatrixNested    MatrixType;
    typedef PetscVectorNested    VectorType;
    
    typedef PetscPreconditionerNested            super;
    typedef PetscPreconPfcDiag                   self;
    typedef PetscRunner<MatrixType, VectorType>  Runner;
    
    struct Creator : CreatorInterfaceName<super>
    {
      virtual ~Creator() {}
      super* create() { return new self(); }
    };

  public:
    PetscPreconPfcDiag()
      : super("pfc_", "pfc_diag"),
	tau(NULL)
    { }

    void setData(double *tauPtr)
    {
      tau = tauPtr;
    }

    void init(PC pc, const SolverMatrix<Matrix<DOFMatrix*> >& A, const MatrixType& fullMatrix);
    void exit();

  private:
    Mat MpK;
    Mat matS;
    PfcDiagData data;

    double *tau;
  };

}

#endif // AMDIS_PETSC_PRECON_PFC_H
